@@ -387,7 +387,7 @@ nlmixrBounds <- function(fun){
                                 n <- n[n != "+"];
                                 stop(sprintf("%s ~ c(%s) does not have the right dimensions for a lower triangular matrix.", paste(n, collapse=" + "), paste(sapply(x[[3]][-1], as.character), collapse=", ")))
                             }
-                        } else if (any(as.character(x[[3]][[1]]) == c("add", "prop"))){
+                        } else if (any(as.character(x[[3]][[1]]) == c("add", "norm", "dnorm", "prop"))){
                             env$theta <- env$theta + 1;
                             env$nerr <- 1;
                             env$df <- rbind(env$df,
@@ -593,6 +593,10 @@ nlmixrBounds <- function(fun){
         stop(sprintf("The lower bound is higher than the estimate for these parameters: %s.\nYou can adjust by %s=c(%s, %s) # c(lower, est)",
                      paste(df$name[w], collapse=", "), df$name[w[1]], df$est[w[1]], df$lower[w[1]]))
     }
+    .w <- which(df$lower == 0)
+    if(length(.w) > 0) df$lower[.w] <- sqrt(.Machine$double.eps)
+    .w <- which(df$upper == 0)
+    if(length(.w) > 0) df$upper[.w] <- -sqrt(.Machine$double.eps)
     class(df) <- c("nlmixrBounds", "data.frame");
     return(df)
 }
@@ -698,8 +702,7 @@ nlmixrBounds.focei.upper.lower <- function(obj, type=c("upper", "lower", "name",
     type <- match.arg(type);
     df <- as.data.frame(obj);
     dft <- df[!is.na(df$ntheta), ];
-    dft.unfixed <- dft[!dft$fix, ];
-    ret <- dft.unfixed[[type]]
+    ret <- dft[[type]]
     if (is(ret, "factor")){
         ret <- paste(ret);
     }
