@@ -10,11 +10,19 @@ RxODE::RxODE
 ##' @export
 RxODE::rxSolve
 
+
+##' @importFrom RxODE rxClean
+##' @export
+RxODE::rxClean
+
+##' @importFrom RxODE rxCat
+##' @export
+RxODE::rxCat
+
 ##' @importFrom nlme nlme
 ##' @export
 nlme::nlme
 
-
 ##' @importFrom nlme ACF
 ##' @export
 nlme::ACF
@@ -23,6 +31,10 @@ nlme::ACF
 ##' @export
 nlme::VarCorr
 
+##' @importFrom nlme getVarCov
+##' @export
+nlme::getVarCov
+
 
 ##' @importFrom nlme augPred
 ##' @export
@@ -36,7 +48,6 @@ nlme::fixef
 ##' @export
 nlme::fixed.effects
 
-
 ##' @importFrom nlme ranef
 ##' @export
 nlme::ranef
@@ -45,8 +56,6 @@ nlme::ranef
 ##' @export
 nlme::random.effects
 
-
-
 ##' @importFrom nlme nlmeControl
 ##' @export
 nlme::nlmeControl
