@@ -22,6 +22,7 @@
 #define __SAEM_CLASS_RCPP_HPP__
 #include <RcppArmadillo.h>
 #include <neldermead.hpp>
+#define MAXENDPNT 40
 using namespace std;
 using namespace arma;
 using namespace Rcpp;
@@ -39,12 +40,27 @@ struct mcmcphi {
 struct mcmcaux {
   int nM;
   uvec indioM;
-  double sigma2;
+  //double sigma2;  //not needed?
   vec yM;
   mat evtM;
   List optM;
 };
 
+
+uvec getObsIdx(umat m) {
+    uvec x;
+	x.set_size(0);
+
+	for (unsigned int b=0; b<m.n_rows; ++b) {
+	    uvec i=linspace<uvec>(m(b,0), m(b,1), m(b,1) - m(b,0) + 1);
+    	x = join_cols(x, i);
+    }
+
+	return x;
+}
+
+
+// class def starts
 class SAEM {
 typedef vec (*user_funct) (const mat&, const mat&, const List&);
 
@@ -60,6 +76,17 @@ void set_fn(user_funct f) {
   user_fn = f;
 }
 
+mat get_resMat(){
+  mat m(nendpnt,2);
+  m.col(0) = ares;
+  m.col(1) = bres;
+  return m;
+}
+mat get_mprior_phi(){
+  mat m = mpost_phi;
+  m.cols(i1) = mprior_phi1;
+  return m;
+}
 mat get_mpost_phi(){
   return mpost_phi;
 }
@@ -73,16 +100,13 @@ mat get_Ha(){
   return Ha;
 }
 vec get_sig2(){
-  vec sig2;
-  sig2 << ares << bres;
-  return sig2;
+  return vcsig2;                                       //FIXME: regression due to multiple endpnts?
 }
 mat get_par_hist(){
   return par_hist;
 }
 
 mat get_eta(){
-  //Rcout << i1;
   mat eta = mpost_phi.cols(i1);
   eta -= mprior_phi1;
   return eta;
@@ -113,8 +137,8 @@ void inits(List x) {
   int mlen = as<int>(x["mlen"]);
   nM = N*nmc;
 
-  opt = as<List>(x["opt"]);		//CHECKME
-  optM = as<List>(x["optM"]);	//CHECKME
+  opt = as<List>(x["opt"]);                                      //CHECKME
+  optM = as<List>(x["optM"]);                                    //CHECKME
 
   pc1 = as<uvec>(x["pc1"]);
   covstruct1 = as<mat>(x["covstruct1"]);
@@ -149,8 +173,6 @@ void inits(List x) {
   }
   fixedIx = as<uvec>(x["fixed.ix"]);
 
-  statrese=0;
-
   nlambda1 = as<int>(x["nlambda1"]);
   nlambda0 = as<int>(x["nlambda0"]);
   nlambda = nlambda1 + nlambda0;
@@ -164,13 +186,30 @@ void inits(List x) {
   phi.set_size(N, nphi, nmc);
 
   //FIXME
-  res_mod = as<int>(x["res.mod"]);
-  //Rcout << "res_mod: " << res_mod << "\n";
-  ares = as<double>(x["ares"]);
-  bres = as<double>(x["bres"]);
-  if (res_mod==1) bres=0;
-  if (res_mod==2) ares=0;
-  sigma2 = max(ares*ares, 10.0); //FIXME
+  nendpnt=as<int>(x["nendpnt"]);
+  ix_sorting=as<uvec>(x["ix_sorting"]);
+  ys = y(ix_sorting);    //ys: obs sorted by endpnt
+  ysM=as<vec>(x["ysM"]);
+  y_offset=as<uvec>(x["y_offset"]);
+  res_mod = as<vec>(x["res.mod"]);
+  ares = as<vec>(x["ares"]);
+  bres = as<vec>(x["bres"]);
+  ix_endpnt=as<uvec>(x["ix_endpnt"]);
+  ix_idM=as<umat>(x["ix_idM"]);
+  res_offset=as<uvec>(x["res_offset"]);
+  nres = res_offset.max();
+  vcsig2.set_size(nres);
+  vecares = ares(ix_endpnt);
+  vecbres = bres(ix_endpnt);
+  for (int b=0; b<nendpnt; ++b) {
+    sigma2[b] = 10.0;
+    if (res_mod(b)==1)
+      sigma2[b] = max(ares(b)*ares(b), 10.0);
+    if (res_mod(b)==2)
+      sigma2[b] = max(bres(b)*bres(b), 1.0);
+
+    statrese[b] = 0.0;
+  }
 
   print = as<int>(x["print"]);
   par_hist = as<mat>(x["par.hist"]);
@@ -192,10 +231,21 @@ void inits(List x) {
   mx.optM = optM;
 
   distribution=as<int>(x["distribution"]);
+  DEBUG=as<int>(x["DEBUG"]);
+  phiMFile=as<std::vector< std::string > >(x["phiMFile"]);
+  //Rcout << phiMFile[0];
+
 }
 
 void saem_fit() {
   //arma_rng::set_seed(99);
+  double double_xmin = 1.0e-200;                               //FIXME hard-coded xmin, also in neldermean.hpp
+  ofstream phiFile;
+  phiFile.open(phiMFile[0].c_str());
+
+  if (DEBUG>0) Rcout << "initialization successful\n";
+  fsave = user_fn(phiM, evtM, optM);
+  if (DEBUG>0) Rcout << "initial user_fn successful\n";
   for (unsigned int kiter=0; kiter<(unsigned int)(niter); kiter++) {
     gamma2_phi1=Gamma2_phi1.diag();
     IGamma2_phi1=inv_sympd(Gamma2_phi1);
@@ -213,7 +263,6 @@ void saem_fit() {
     mcmcphi mphi1, mphi0;
     set_mcmcphi(mphi1, i1, nphi1, Gamma2_phi1, IGamma2_phi1, mprior_phi1);
     set_mcmcphi(mphi0, i0, nphi0, Gamma2_phi0, IGamma2_phi0, mprior_phi0);
-    mx.sigma2 = sigma2;
 
     // CHG hard coded 20
     int nu1, nu2, nu3;
@@ -223,17 +272,21 @@ void saem_fit() {
       nu1=nu(0); nu2=nu(1); nu3=nu(2);
     }
 
-    vec f=user_fn(phiM, evtM, optM);
-    vec g = ares + bres*f;
+    vec f = fsave;
+    vec g = vecares + vecbres % abs(f);                          //make sure g > 0
+    g.elem( find( g < double_xmin) ).fill(double_xmin);
+
+    //fsave = f;
     if (distribution == 1) DYF(indioM)=0.5*(((yM-f)/g)%((yM-f)/g))+log(g);
-    else 
+    else
     if (distribution == 2) DYF(indioM)=-yM%log(f)+f;
     else
     if (distribution == 3) DYF(indioM)=-yM%log(f)-(1-yM)%log(1-f);
     else {
-		Rcout << "unknown distribution\n";
-		return;
+        Rcout << "unknown distribution\n";
+        return;
     }
+    //U_y is a vec of subject llik; summed over obs for each subject
     vec U_y=sum(DYF,0).t();
 
     if(nphi1>0) {
@@ -252,18 +305,23 @@ void saem_fit() {
       do_mcmc(2, nu2, mx, mphi0, DYF, phiM, U_y, U_phi);
       do_mcmc(3, nu3, mx, mphi0, DYF, phiM, U_y, U_phi);
     }
+    if (DEBUG>0) Rcout << "mcmc successful\n";
+    phiFile << phiM;
+    //mat dphi=phiM.cols(i1)-mphi1.mprior_phiM;
+    //vec U_phi=0.5*sum(dphi%(dphi*IGamma2_phi1),1);
 
     //  MCMC stochastic approximation
     mat Statphi11=zeros<mat>(N,nphi1);
     mat Statphi01=zeros<mat>(N,nphi0);
     mat Statphi12=zeros<mat>(nphi1,nphi1);
     mat Statphi02=zeros<mat>(nphi0,nphi0);
-    double statr=0, resk=0;
+    double statr[MAXENDPNT], resk;
+    for(int b=0; b<nendpnt; ++b) statr[b]= 0;
 
     vec D1=zeros<vec>(nb_param);    //CHG!!!
     mat D11=zeros<mat>(nb_param,nb_param);
     mat D2=zeros<mat>(nb_param,nb_param);
-    vec resy(nmc); //FIXME
+    vec resy(nmc);
     mat d2logk=zeros<mat>(nb_param,nb_param);
 
     d2logk(span(0,nlambda1-1),span(0,nlambda1-1))=-CGamma21;
@@ -271,6 +329,10 @@ void saem_fit() {
     d2logk(span(nlambda1,nlambda-1),span(nlambda1,nlambda-1))=-CGamma20;
     }
 
+
+    vec fsM;
+    fsM.set_size(0);
+    //integration
     for(int k=0; k<nmc; k++) {
       phi.slice(k)=phiM.rows(span(k*N,(k+1)*N-1));
 
@@ -282,35 +344,52 @@ void saem_fit() {
       Statphi12=Statphi12+phi1k.t()*phi1k;
       Statphi02=Statphi02+phi0k.t()*phi0k;
 
-      vec fk=user_fn(phik, evt, opt);
-      vec gk;
+      vec fk = fsave(span(k*ntotal, (k+1)*ntotal-1));
+      fk = fk(ix_sorting);    //sorted by endpnt
+      fsM = join_cols(fsM, fk);
+      vec resid_all = ys - fk;
+      vec gk, resid;
 
-      if (res_mod==1) gk.ones();
-      if (res_mod==2) {
-		gk = fk;
-		gk.elem( find(gk < 1e-8) ).fill(1e-8);  //FIXME by eps
+      //loop thru endpoints here
+      for(int b=0; b<nendpnt; ++b) {
+        resid = resid_all(span(y_offset(b),y_offset(b+1)-1));
+        if (res_mod(b)==2) {
+        //double epsilon = std::numeric_limits<double>::epsilon();
+          gk = abs(fk(span(y_offset(b),y_offset(b+1)-1)));            //CHK: range & chk resize & .memptr()
+          gk.elem( find( gk < double_xmin) ).fill(double_xmin);
+          resid = resid/gk;
+        }
+#if 0
+        uvec iix = find(resid>1e9);
+        Rcout << b << " " <<iix;
+        Rcout << ys(iix) << fk(iix) << gk(iix);
+#endif
+
+        if (res_mod(b)<=2)
+          resk = dot(resid, resid);
+        else
+          resk = 1;                                              //FIXME
+
+        statr[b]=statr[b]+resk;
+        resy(k) = resk;                                          //FIXME: resy(b,k)?
       }
-      if (res_mod==1) resk=dot(y-fk, y-fk);
-      else if (res_mod==2) resk=dot((y-fk)/gk, (y-fk)/gk);
-      else resk = 1;	//FIXME
-      statr=statr+resk;
-      resy(k) = resk;
+      if (DEBUG>1) Rcout << "star[] successful\n";
 
       mat dphi1k=phi1k-mprior_phi1;
       mat dphi0k=phi0k-mprior_phi0;
       vec sdg1=sum(dphi1k%dphi1k,0).t()/gamma2_phi1;
       mat Md1=(IGamma2_phi1*(dphi1k.t()*Mcovariables)).t();
       mat Md0=(IGamma2_phi0*(dphi0k.t()*Mcovariables)).t();
-      vec d1_mu_phi1=Md1(ind_cov1);    //CHK!! vec or mat
-      vec d1_mu_phi0=Md0(ind_cov0);    //CHK!! vec or mat
+      vec d1_mu_phi1=Md1(ind_cov1);                              //CHK!! vec or mat
+      vec d1_mu_phi0=Md0(ind_cov0);                              //CHK!! vec or mat
       vec d1_loggamma2_phi1=0.5*sdg1-0.5*N;
       vec d1_logsigma2;
-      d1_logsigma2 << 0.5*resy(k)/sigma2-0.5*ntotal;
+      d1_logsigma2 << 0.5*resy(k)/sigma2[0]-0.5*ntotal;          //FIXME: sigma2[0], sigma2[b] instead?
       vec d1logk=join_cols(d1_mu_phi1, join_cols(d1_mu_phi0, join_cols(d1_loggamma2_phi1, d1_logsigma2)));
       D1 = D1+d1logk;
       D11= D11+d1logk*d1logk.t();
 
-      vec w2phi=-0.5*sdg1;    //CHK!!!
+      vec w2phi=-0.5*sdg1;                                       //CHK!!!
       for(int j=0, l=0; j<nphi1; j++) {
         for(unsigned int jj=0; jj<pc1(j); jj++) {
           double temp=-dot(COV1.col(l),dphi1k.col(j))/gamma2_phi1(j);
@@ -320,15 +399,17 @@ void saem_fit() {
         }
         d2logk(nlambda+j,nlambda+j)=w2phi(j);
       }
-      d2logk(nb_param-1,nb_param-1)=-0.5*resy(k)/sigma2;
+      d2logk(nb_param-1,nb_param-1)=-0.5*resy(k)/sigma2[0];      //FIXME: sigma2[0], sigma2[b] instead?
       D2=D2+d2logk;
-    }
+    }//k
+    if (DEBUG>0) Rcout << "integration successful\n";
 
     statphi11=statphi11+pas(kiter)*(Statphi11/nmc-statphi11);
     statphi12=statphi12+pas(kiter)*(Statphi12/nmc-statphi12);
     statphi01=statphi01+pas(kiter)*(Statphi01/nmc-statphi01);
     statphi02=statphi02+pas(kiter)*(Statphi02/nmc-statphi02);
-    statrese=statrese+pas(kiter)*(statr/nmc-statrese);
+    for(int b=0; b<nendpnt; ++b)
+      statrese[b]=statrese[b]+pas(kiter)*(statr[b]/nmc-statrese[b]);
 
     // update parameters
     vec Plambda1, Plambda0;
@@ -338,12 +419,12 @@ void saem_fit() {
     Plambda0=inv_sympd(CGamma20)*sum((D1Gamma20%(COV0.t()*statphi01)),1);
     if (fixedIx.n_elem>0) {
       Plambda0(fixedIx) = MCOV0(jcov0(fixedIx));
-	}
+    }
     MCOV0(jcov0)=Plambda0;
     }
     mprior_phi1=COV1*MCOV1;
     mprior_phi0=COV0*MCOV0;
-    mprior_phi0.set_size(N, nphi0);		// deal w/ nphi0=0
+    mprior_phi0.set_size(N, nphi0);                              // deal w/ nphi0=0
 
     mat G1=statphi12/N+mprior_phi1.t()*mprior_phi1/N - statphi11.t()*mprior_phi1/N - mprior_phi1.t()*statphi11/N;
     if (kiter<=(unsigned int)(nb_sa))
@@ -368,27 +449,46 @@ void saem_fit() {
       dGamma2_phi0=Gamma2_phi0.diag();
       } else
       dGamma2_phi0=dGamma2_phi0*coef_phi0;
-      Gamma2_phi0=diagmat(dGamma2_phi0);    //CHK
+      Gamma2_phi0=diagmat(dGamma2_phi0);                         //CHK
     }
 
-    double sig2=statrese/ntotal;
-    if (res_mod==1) ares=sqrt(sig2);
-    else if (res_mod==2) bres=sqrt(sig2);
-    else {
-      yptr = yM.memptr();
-      fptr = f.memptr();
-      len = ntotal;
-      vec xmin(2);
-      double *pxmin = xmin.memptr();
+    //CHECK the following seg on b & yptr & fptr
+    for(int b=0; b<nendpnt; ++b) {
+      double sig2=statrese[b]/(y_offset(b+1)-y_offset(b));       //CHK: range
+      if (res_mod(b)==1)
+        ares(b) = sqrt(sig2);
+      else
+      if (res_mod(b)==2)
+        bres(b) = sqrt(sig2);
+      else {
+        uvec idx;
+        idx = find(ix_endpnt==b);
+        vec ysb, fsb;
 
-      int n=2, itmax=500, iconv, it, nfcall, iprint=0;
-      double start[2]={ares,bres}, step[2]={-.2,-.2}, ynewlo;
-      nelder_(obj, n, start, step, itmax, 1.0e-6, 1.0, 2.0, .5,
-        &iconv, &it, &nfcall, &ynewlo, pxmin, &iprint);
-      ares = ares+pas(kiter)*(pxmin[0]-ares);
-      bres = bres+pas(kiter)*(pxmin[1]-bres);
+        ysb = ysM(idx);
+        fsb = fsM(idx);
+
+        yptr = ysb.memptr();
+        fptr = fsb.memptr();
+        len = ysb.n_elem;                                        //CHK: needed by nelder
+        vec xmin(2);
+        double *pxmin = xmin.memptr();
+        int n=2, itmax=50, iconv, it, nfcall, iprint=0;
+        double start[2]={sqrt(ares(b)), sqrt(fabs(b))};                  //force are & bres to be positive
+        double step[2]={-.2, -.2}, ynewlo;
+        nelder_(obj, n, start, step, itmax, 1.0e-4, 1.0, 2.0, .5,        //CHG hard-coded tol
+                &iconv, &it, &nfcall, &ynewlo, pxmin, &iprint);
+        ares(b) = ares(b) + pas(kiter)*(pxmin[0]*pxmin[0] - ares(b));    //force are & bres to be positive
+        bres(b) = bres(b) + pas(kiter)*(pxmin[1]*pxmin[1] - bres(b));    //force are & bres to be positive
+      }
+      sigma2[b] = sig2;                                          //CHK: sigma2[] use
+      if (sigma2[b]>1.0e99) sigma2[b] = 1.0e99;
+      if (isnan(sigma2[b])) sigma2[b] = 1.0e99;
     }
-    sigma2=sig2;	//FIXME
+    vecares = ares(ix_endpnt);
+    vecbres = bres(ix_endpnt);
+    if (DEBUG>0) Rcout << "par update successful\n";
+
 
     //    Fisher information
     DDa=(D1/nmc)*(D1/nmc).t()-D11/nmc-D2/nmc;
@@ -403,10 +503,16 @@ void saem_fit() {
     cpost_phi=cpost_phi+pash(kiter)*(sphi2/nmc-cpost_phi);
     mpost_phi.cols(i0)=mprior_phi0;
 
-    vec vcsig2;
-    if (res_mod==1) vcsig2 << sigma2;
-    if (res_mod==2) vcsig2 << bres;
-    if (res_mod==3) vcsig2 << ares << bres;
+    //FIXME: chg according to multiple endpnts; need to chg dim(par_hist)
+    for (int b=0; b<nendpnt; ++b) {
+		int offset = res_offset[b];
+        if (res_mod(b)==1) vcsig2[offset] = sigma2[b];
+        if (res_mod(b)==2) vcsig2[offset] = bres(b);
+        if (res_mod(b)==3) {
+			vcsig2[offset]   = ares(b);
+			vcsig2[offset+1] = bres(b);
+		}
+	}
 
     Plambda(ilambda1) = Plambda1;
     Plambda(ilambda0) = Plambda0;
@@ -418,6 +524,8 @@ void saem_fit() {
           << par_hist.row(kiter);
     Rcpp::checkUserInterrupt();
   }//kiter
+  phiFile.close();
+
 }
 
 
@@ -439,7 +547,7 @@ private:
   int nM;
 
   int ntotal, N;
-  vec y, yM;
+  vec y, yM, ys;    //ys is y sorted by endpnt
   mat evt, evtM;
   mat phiM;
   uvec indioM;
@@ -464,9 +572,11 @@ private:
   uvec ilambda1, ilambda0;
 
   mat statphi01, statphi02, statphi11, statphi12;
-  double statrese;
-  double sigma2, ares, bres;
-  int res_mod;
+  double statrese[MAXENDPNT];
+  double sigma2[MAXENDPNT];
+  vec ares, bres;
+  vec vecares, vecbres;
+  vec res_mod;
 
   mat DYF;
   cube phi;
@@ -481,6 +591,20 @@ private:
   mat par_hist;
   int distribution;
 
+  int nendpnt;
+  uvec ix_endpnt;
+  umat ix_idM;
+  uvec y_offset;
+  uvec res_offset;
+  vec vcsig2;
+  int nres;
+  uvec ix_sorting;
+  vec ysM;
+  vec fsave;
+
+  int DEBUG;
+  std::vector< std::string > phiMFile;
+
 void set_mcmcphi(mcmcphi &mphi1,
                  const uvec i1,
                  const int nphi1,
@@ -506,8 +630,9 @@ void do_mcmc(const int method,
              vec &U_phi) {
   vec fc, Uc_y, Uc_phi, deltu;
   uvec ind;
-  vec gc;  //FIXME
+  vec gc;
   uvec i=mphi.i;
+  double double_xmin = 1.0e-200;                               //FIXME hard-coded xmin, also in neldermean.hpp
 
   for (int u=0; u<nu; u++)
   for (int k1=0; k1<mphi.nphi; k1++) {
@@ -520,10 +645,11 @@ void do_mcmc(const int method,
     if (method==3)
       phiMc.col(i(k1))=phiM.col(i(k1))+randn<vec>(mx.nM)*mphi.Gdiag_phi(k1,k1);
 
-    fc=user_fn(phiMc, mx.evtM, mx.optM);
-    gc = ares + bres*fc;
+    fc = user_fn(phiMc, mx.evtM, mx.optM);
+    gc = vecares + vecbres % abs(fc);                            //make sure gc > 0
+    gc.elem( find( gc < double_xmin) ).fill(double_xmin);
     if (distribution == 1) DYF(mx.indioM)=0.5*(((mx.yM-fc)/gc)%((mx.yM-fc)/gc))+log(gc);
-    else 
+    else
     if (distribution == 2) DYF(mx.indioM)=-mx.yM%log(fc)+fc;
     else
     if (distribution == 3) DYF(indioM)=-mx.yM%log(fc)-(1-mx.yM)%log(1-fc);
@@ -540,15 +666,29 @@ void do_mcmc(const int method,
     phiM(ind,i)=phiMc(ind,i);
     U_y(ind)=Uc_y(ind);
     if (method>1) U_phi(ind)=Uc_phi(ind);
+    ind = getObsIdx(ix_idM.rows(ind));
+    fsave(ind)=fc(ind);
     if (method<3) break;
   }
 }
 };
 
+
+// closing for #ifndef __SAEM_CLASS_RCPP_HPP__
 #endif
 
+
+
 /*
- * prop err
- * combo err
- * clean up sigma2 related code
- */
+- allow phi to psi xform
+- chg g<EPS & gc<EPS, chg to EPS
+- distribution & res_mod be nested
+- clean up sigma2 related code
+- chk & fix resy(k) for multiple endpnts
+    * resy(k) is only used in Fisher info concerning sigma2; currently NOT used by nlmixr
+- two for loops to convert double to integer for EVID & CMT
+    * may declare a umat to hold these to avoid loops for faster speed
+    * examine all loops
+*/
+
+
