@@ -56,28 +56,28 @@ rxPermissive({
                 response.scaler = "V",
                 verbose = TRUE,
                 weight = varPower(fixed = c(1)),
-                control = nlmeControl(pnlsTol = .1, msVerbose = TRUE)
+                control = nlmeControl(pnlsTol = .1, msVerbose = TRUE, msMaxIter=1000)
             )
 
         z <- VarCorr(fit)
 
-        expect_equal(signif(as.numeric(fit$logLik), 6),-11754.6)
-        expect_equal(signif(AIC(fit), 6), 23535.1)
-        expect_equal(signif(BIC(fit), 6), 23609.7)
+        expect_equal(signif(as.numeric(fit$logLik), 6), -11769.1)
+        expect_equal(signif(AIC(fit), 6), 23564.2)
+        expect_equal(signif(BIC(fit), 6), 23638.7)
 
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[1]), 3), 7.3)
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[2]), 3), 6.07)
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[3]), 3), 4.26)
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[4]), 3), 1.33)
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[5]), 3), 3.73)
-        expect_equal(signif(as.numeric(fit$coefficients$fixed[6]), 3), -0.00552)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[1]), 3), 7.32)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[2]), 3), 6.09)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[3]), 3), 4.25)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[4]), 3), 1.38)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[5]), 3), 3.75)
+        expect_equal(signif(as.numeric(fit$coefficients$fixed[6]), 3), -0.021)
 
-        expect_equal(signif(as.numeric(z[1, "StdDev"]), 3),  0.188)
-        expect_equal(signif(as.numeric(z[2, "StdDev"]), 3), 0.335)
-        expect_equal(signif(as.numeric(z[3, "StdDev"]), 3), 0.339)
-        expect_equal(signif(as.numeric(z[4, "StdDev"]), 3), 0.000319)
-        expect_equal(signif(as.numeric(z[5, "StdDev"]), 3), 0.254)
-        expect_equal(signif(as.numeric(z[6, "StdDev"]), 3), 0.286)
+        expect_equal(signif(as.numeric(z[1, "StdDev"]), 3),  0.19)
+        expect_equal(signif(as.numeric(z[2, "StdDev"]), 3), 0.33)
+        expect_equal(signif(as.numeric(z[3, "StdDev"]), 3), 0.336)
+        expect_equal(signif(as.numeric(z[4, "StdDev"]), 3), 0.000205)
+        expect_equal(signif(as.numeric(z[5, "StdDev"]), 3), 0.253)
+        expect_equal(signif(as.numeric(z[6, "StdDev"]), 3), 0.283)
 
         expect_equal(signif(fit$sigma, 3), 0.205)
     })
